/*  
  Copyright (C) 2005 Giulio Bottazzi

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  (version 2) as published by the Free Software Foundation;
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "tools.h"


int my_atoi(const char *string){

  char *end;
  int res = strtol(string,&end, 10);
  
  if(string[0] == '\0'){
    fprintf(stderr,"my_atoi: conversion of empty string\n");
    exit(2);
  }
  else if(*end != '\0'){
    fprintf(stderr,"my_atoi: error in converting %s to integer\n",string);
    exit(2);    
  }

  return res;

}

void *my_alloc(const size_t size){
    void *temp;
    if(!(temp = malloc(size))){
	perror("malloc, memory allocation failed");
	exit(1);
    }
    return(temp);
}

void *my_realloc(void *ptr,const size_t size){
  void *temp=NULL;
  if(size == 0){
    free(ptr);
  }
  else if(!(temp = realloc(ptr,size))){
    perror("realloc, memory allocation failed");
    exit(1);
  }
  return(temp);
}


void testfpe(void){
  int raised = fetestexcept (FE_DIVBYZERO | FE_UNDERFLOW |
			       FE_OVERFLOW | FE_INVALID);

  if( raised & (FE_DIVBYZERO | FE_UNDERFLOW |FE_OVERFLOW | FE_INVALID) ){
    fprintf(stderr,"WARNING: floating point exception raised:");

    if(raised &  FE_DIVBYZERO)
      fprintf(stderr," FE_DIVBYZERO");
    
    if(raised & FE_UNDERFLOW)
      fprintf(stderr," FE_UNDERFLOW");
    
    if(raised & FE_OVERFLOW)
      fprintf(stderr," FE_OVERFLOW");
    
    if(raised & FE_INVALID)
      fprintf(stderr," FE_INVALID");
    
    fprintf(stderr,"\n");
  }

  if( feclearexcept (FE_INEXACT | FE_DIVBYZERO | FE_UNDERFLOW |
		     FE_OVERFLOW | FE_INVALID) ) {
    fprintf(stderr,"ERROR: unable to reset floating point exception\n");
    exit(-1);
  }

}

double my_max (double a, double b){
  return a > b ? a : b;
}

double my_min (double a, double b){
  return a > b ? b : a;
}
